package no.ntnu.wifimanager.activity;

import static no.ntnu.wifimanager.CommonUtilities.LOG_TAG;
import static no.ntnu.wifimanager.CommonUtilities.SERVER_URL;

import java.io.IOException;
import java.io.UnsupportedEncodingException;

import no.ntnu.wifimanager.R;
import no.ntnu.wifimanager.domain.Client;
import no.ntnu.wifimanager.domain.mapping.ClientMapper;
import no.ntnu.wifimanager.domain.permission.AccessPermission;
import no.ntnu.wifimanager.view.AccessPermissionSelector;
import no.ntnu.wifimanager.view.AccessPermissionSelector.AccessPermissionStatus;

import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicHeader;
import org.apache.http.protocol.HTTP;

import android.app.ActionBar;
import android.app.Activity;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.TextView;


public abstract class BaseClientActivity extends Activity {


	private static final String URL_POST_CLIENT_PERMS = SERVER_URL+"/postclient.php";
	protected Client client;
	protected ActionBar actionBar;
	protected TextView allowOrBlockView;
	protected AccessPermissionSelector accessPermSelector;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_client_permissions);

		client = new Client();
		actionBar = getActionBar();
		configureViews();
	}

	protected void configureViews() {

		accessPermSelector = (AccessPermissionSelector) findViewById(R.id.access_perm_selector);
		allowOrBlockView = (TextView) findViewById(R.id.allow_or_block_text_view);

		accessPermSelector.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				accessPermSelector.showNextStatus();
			}
		});
	}



	public void confirmPermissions(View view){
		readUserInputToClient();
		String client_json = ClientMapper.toJSONClient(client).toString();
		new PostClientTask().execute(URL_POST_CLIENT_PERMS, client_json);
	}


	public void cancelPermissions(View view){
		finishActivityWithResult(false);
	}


	private void finishActivityWithResult(boolean clientUpdated) {

		Intent intent = getIntent();

		if(clientUpdated){
			
			setResult(ClientListActivity.RESULT_CLIENT_UPDATE, intent);
	
		}else{
			setResult(ClientListActivity.RESULT_NO_CLIENT_UPDATE, intent);
		}
		
		super.finish();
	}

	
	private void readUserInputToClient() {
		
		readAccessPermissionInput();
	
	}

	private void readAccessPermissionInput() {

		AccessPermissionStatus status = accessPermSelector.getStatus();
		AccessPermission accssPerm = new AccessPermission();

		switch (status) {
		case ACCESS_ALLOW:
			accssPerm.isAccessAllowed(true);
			client.setAccessPermission(accssPerm);
			break;

		case ACCESS_BLOCK:
			accssPerm.isAccessAllowed(false);
			client.setAccessPermission(accssPerm);
			break;
		}
	}


	protected class PostClientTask  extends AsyncTask<String, Void, String>{

		@Override
		protected String doInBackground(String... params) {
			try {
				HttpPost httpPost = new HttpPost(params[0]);
				StringEntity se = new StringEntity(params[1]);
				se.setContentType("application/json");
				se.setContentEncoding(new BasicHeader(HTTP.CONTENT_TYPE,"application/json"));
				//httpPost.setHeader("Accept", "application/json");
				httpPost.setEntity(se);
				Log.d(LOG_TAG, "client sent to server: "+ params[1]);
				HttpResponse resp = new DefaultHttpClient().execute(httpPost);
				Log.d(LOG_TAG, resp.getStatusLine().toString());
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}

			catch (ClientProtocolException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
			return null;
		}

		@Override
		protected void onPostExecute(String result) {
			super.onPostExecute(result);
			BaseClientActivity.this.finishActivityWithResult(true);
		}
	}

}
